<?php

namespace ZahlsPaymentGateway\Webhook;

use ZahlsPaymentGateway\Controller\OrderService;
use ZahlsPaymentGateway\Service\ZahlsApiService;

class Dispatcher
{
    /**
     * @var ZahlsApiService
     */
    private $zahlsApiService;
    /**
     * @var OrderService
     */
    private $orderService;

    public function __construct($zahlsApiService, $orderService)
    {
        $this->zahlsApiService = $zahlsApiService;
        $this->orderService = $orderService;
    }

    public function check_webhook_response()
    {
        try {
            $resp = $_REQUEST;
            $orderId = $resp['transaction']['invoice']['referenceId'];

            if (!empty($this->prefix) && strpos($orderId, $this->prefix) === false) {
                return;
            }

            $arr = explode('_', $orderId);
            $orderId = end($arr);

            if (!isset($resp['transaction']['status'])) {
                throw new \Exception('Missing transaction status');
            }

            /**
             * @var \Zahls\Models\Response\Transaction
             */
            $transaction = $this->zahlsApiService->getZahlsTransaction($resp['transaction']['id']);

            if ($transaction->getStatus() !== $resp['transaction']['status']) {
                throw new \Exception('Fraudulent transaction status');
            }

            // Check if subscription to handle accordingly
            $subscriptions = [];
            if (!empty($resp['transaction']['preAuthorizationId'])) {
                $subscriptions = wcs_get_subscriptions_for_order($orderId, array( 'order_type' => 'any' ));

                // $orderId is the subscription id in case of payment method change. In this case $subscriptions will be empty
                if (!$subscriptions) {
                    $subscriptions[] = new \WC_Subscription($orderId);
                }

                // Identify the correct orderId
                // Automatic subscription payment > orderId is already valid and matches the referenceId. It must not be overwritten
                // Payment method change > $orderId is a subscriptionId and must be overwritten
                // Subscription renewal > $orderId is from an old order and must be overwritten
                $firstSubscription = reset($subscriptions);
                $orderId = $firstSubscription->get_last_order();
            }

            $order = new \WC_Order($orderId);

            if (!$order) {
                throw new \Exception('Fraudulent request');
            }

            $this->orderService->handleTransactionStatus($order, $subscriptions, $transaction, $resp['transaction']['preAuthorizationId']);

        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
}